local E, L, V, P, G = unpack(ElvUI);
local mPlugin = "mMediaTag"
local DT = E:GetModule("DataTexts");
local addon, ns = ...
local displayString = ""
local mText = L["Professions"]

local menuFrame = CreateFrame("Frame", "mProfessionMenu", E.UIParent)
menuFrame:SetTemplate("Transparent", true)

local prof1, prof2, archaeology, fishing, cooking, firstAid = nil
local name, icon, skillLevel, maxSkillLevel, spelloffset, skillLine, skillModifier, specializationIndex, specializationOffset = nil
local mI = 0
local menuList = {}

local function getProf(index, isDT)
	local mskillModifier = 0
	local mIcon = E.db[mPlugin].mProfIcon
	name, icon, skillLevel, maxSkillLevel, _, _, skillLine, skillModifier, _, _ = GetProfessionInfo(index)
	
	if skillModifier ~= 0 then mskillModifier = " +" .. skillModifier else mskillModifier = "" end
	
	if isDT then
		if (skillLevel == maxSkillLevel) then
			return format ("(" .. ns.mColor6 .. "" .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "|r)")
		else
			return format ("(" .. ns.mColor2 .. "" .. skillLevel .. "|r" .. ns.mColor6 .. mskillModifier .. "/" .. ns.mColor7 .. "" .. maxSkillLevel .. "|r)")
		end
	else
		if mIcon then
			if (skillLevel == maxSkillLevel) then
				return format ("|T"..icon..":0|t  " .. name)
			else
				return format ("|T"..icon..":0|t  " .. name)
			end
		else
			if (skillLevel == maxSkillLevel) then
				return format (name)
			else
				return format (name)
			end
		end
	end
end
 
local function castProf(index)
	name, icon, skillLevel, maxSkillLevel, _, spelloffset, skillLine, skillModifier, specializationIndex, specializationOffset  = GetProfessionInfo(index)

	CastSpell(spelloffset + 1, "Spell")
end

local function setupList(index, color, text, skill, title, spell)
	if spell == nil then
		tinsert(menuList, index, {text = color .. text, DTskill = skill, isTitle = title, func = function() end})
	else
		tinsert(menuList, index, {text = color .. text, DTskill = skill, isTitle = title, func = function() castProf(spell) end})
	end
end

local function setupProf()
	prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
	mI = 1
	menuList = {}
	
	if (prof1 == nil) and (prof2 == nil) then
		setupList(mI, ns.mColor5, L["No Main Professions|r"], nil, true, nil)
		mI = mI + 1
	else
		setupList(mI, ns.mColor2, L["Main Professions|r"], nil, true, nil)
		mI = mI + 1
		
		if prof1 ~= nil then
			setupList(mI, ns.mColor1, getProf(prof1, false), getProf(prof1, true), false, prof1)
			mI = mI + 1
		end
	
		if prof2 ~= nil then
			setupList(mI, ns.mColor1, getProf(prof2, false), getProf(prof2, true), false, prof2)
			mI = mI + 1
		end
	end
	
	setupList(mI, ns.mColor1, "|r", nil, true, nil)
	mI = mI + 1
	
	if  (archaeology == nil) and (cooking == nil) and (firstAid == nil) and (fishing == nil)  then
		setupList(mI, ns.mColor5, L["No Secondary Professions|r"], nil, true, nil)
		mI = mI + 1
	else
		setupList(mI, ns.mColor2, L["Secondary Professions|r"], nil, true, nil)
		mI = mI + 1
		
		if archaeology ~= nil then
			setupList(mI, ns.mColor1, getProf(archaeology, false), getProf(archaeology, true), false, archaeology)
			mI = mI + 1
		end
		
		if cooking ~= nil then
			setupList(mI, ns.mColor1, getProf(cooking, false), getProf(cooking, true), false, cooking)
			mI = mI + 1
		end
		
--[[ 		if firstAid ~= nil then
			setupList(mI, ns.mColor1, getProf(firstAid, false), getProf(firstAid, true), false, firstAid)
			mI = mI + 1
		end ]]
		
		if fishing ~= nil then
			setupList(mI, ns.mColor1, getProf(fishing, false), getProf(fishing, true), false, fishing)
			mI = mI + 1
		end
	end
end

local function mSetupTooltip ()
	DT.tooltip:AddLine(mText)
	DT.tooltip:AddLine(L["   "])
	
	setupProf()
	if prof1 ~= nil then
		DT.tooltip:AddDoubleLine(menuList[2].text, menuList[2].DTskill, 1, 1, 1, 1, 1, 0)
	end
	if prof2 ~= nil then
		DT.tooltip:AddDoubleLine(menuList[3].text, menuList[3].DTskill, 1, 1, 1, 1, 1, 0)
	end
	if archaeology ~= nil then
		DT.tooltip:AddDoubleLine(menuList[6].text, menuList[6].DTskill, 1, 1, 1, 1, 1, 0)
	end
	if cooking ~= nil then
		DT.tooltip:AddDoubleLine(menuList[7].text, menuList[7].DTskill, 1, 1, 1, 1, 1, 0)
	end
--[[ 	if firstAid ~= nil then
		DT.tooltip:AddDoubleLine(menuList[8].text, menuList[8].DTskill, 1, 1, 1, 1, 1, 0)
	end ]]
	if fishing ~= nil then
		DT.tooltip:AddDoubleLine(menuList[8].text, menuList[8].DTskill, 1, 1, 1, 1, 1, 0)
	end
	if fishing == nil and cooking == nil and archaeology == nil and prof2 == nil and prof1 == nil then
		DT.tooltip:AddLine(ns.mColor5 .. L["No Professions|r"])
	end
	
	DT.tooltip:AddLine(L["   "])
	DT.tooltip:AddLine(ns.mColor8 .. L["right click to open the profession window"] .. "|r")
	DT.tooltip:AddLine(ns.mColor8 .. L["left click to open the menu."] .. "|r")
end

local function OnClick(self, button)
	DT.tooltip:Hide()
	if button == "LeftButton" then
		local mProfSetting = E.db[mPlugin].mProfUp
		setupProf()
		mDropDown(menuList, menuFrame, mProfSetting, 200)
	else
		ToggleSpellBook(BOOKTYPE_PROFESSION)
	end
end
	
local function OnEnter(self)
	DT:SetupTooltip(self)
	mSetupTooltip()
	DT.tooltip:Show()
end

local function OnEvent(self, event, unit)
	self.text:SetFormattedText(displayString, mText)
end

local function OnLeave(self)
	DT.tooltip:Hide()
end

local function ValueColorUpdate(hex)
	displayString = strjoin("", hex, "%s|r")

	if lastPanel ~= nil then
		OnEvent(lastPanel, "ELVUI_COLOR_UPDATE")
	end
end
E.valueColorUpdateFuncs[ValueColorUpdate] = true

DT:RegisterDatatext("Professions", nil, OnEvent, nil, OnClick, OnEnter, OnLeave, mText)